// SuDoku.cpp: implementation of the CSuDoku class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SuDokuSolver.h"
#include "SuDoku.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// just a reminder to self: ROW is a Y coordinate; COLUMN is an X coordinate.

CSuDoku::CSuDoku()
{
	ClearEverything();
}

CSuDoku::~CSuDoku()
{

}

void CSuDoku::ClearEverything()
{
	int x, y, digit;
	for (x=0; x<9; ++x) {
		for (y=0; y<9; ++y) {
			m_iStartingValues[x][y] = DIGIT_UNKNOWN;
			m_iKnownValues[x][y] = DIGIT_UNKNOWN;
			m_bAllowedToGuess[x][y] = true;
			m_bPossible[x][y][0] = false; // wipe digit '0' too
			for (digit=0; digit<10; ++digit) {
				m_bPossible[x][y][digit] = true;
			}
		}
	}
	m_iRecursionLevel = 0;
}

void CSuDoku::InitializeXY(int iStartingValues[9][9])
{
	int x, y, digit;
	for (x=0; x<9; ++x) {
		for (y=0; y<9; ++y) {
			m_iStartingValues[x][y] = iStartingValues[x][y];
			m_iKnownValues[x][y] = iStartingValues[x][y];
			m_bAllowedToGuess[x][y] = true;
			for (digit=1; digit<10; ++digit) {
				m_bPossible[x][y][digit] = (m_iKnownValues[x][y] == DIGIT_UNKNOWN) || (m_iKnownValues[x][y] == digit);
			}
		}
	}
	m_iRecursionLevel = 0;
}

void CSuDoku::InitializeYX(int iStartingValues[9][9])
{
	int x, y, digit;
	for (x=0; x<9; ++x) {
		for (y=0; y<9; ++y) {
			m_iStartingValues[x][y] = iStartingValues[y][x]; // note difference from InitializeXY
			m_iKnownValues[x][y] = iStartingValues[y][x]; // note difference from InitializeXY
			m_bAllowedToGuess[x][y] = true;
			for (digit=1; digit<10; ++digit) {
				m_bPossible[x][y][digit] = (m_iKnownValues[x][y] == DIGIT_UNKNOWN) || (m_iKnownValues[x][y] == digit);
			}
		}
	}
	m_iRecursionLevel = 0;
}

bool CSuDoku::IsComplete()
{
	UpdateKnownValues();
	return m_bIsComplete;
}

bool CSuDoku::IsConsistent()
{
	UpdateKnownValues();
	return m_bIsConsistent;
}

void CSuDoku::UpdateKnownValues()
{
	int x, y, i, digit, numpossible, onlypossibledigit;
	bool bXContains[9][10]; // x (0-8), digit (1-9)
	bool bYContains[9][10]; // y (0-8), digit (1-9)
	bool bSquareContains[9][10]; // square (0-8), digit (1-9);
	m_bIsComplete = true;
	m_bIsConsistent = true;

	// initialize "X/Y/square contains..." arrays to false;
	for (i=0; i<9; ++i) {
		for (digit=1; digit<10; ++digit) {
			bXContains[i][digit] = false;
			bYContains[i][digit] = false;
			bSquareContains[i][digit] = false;
		}
	}
	for (x=0; x<9; ++x) {
		for (y=0; y<9; ++y) {
			m_iKnownValues[x][y] = DIGIT_UNKNOWN; // presume unknown...
			// unless only one possible value:
			numpossible = 0;
			for (digit=1; digit<10; ++digit) {
				if (m_bPossible[x][y][digit]) {
					++numpossible;
					onlypossibledigit = digit;
				}
			}
			if (numpossible == 1) {
				m_iKnownValues[x][y] = onlypossibledigit;
				if (bXContains[x][onlypossibledigit]) m_bIsConsistent = false; // already has one of these!
				bXContains[x][onlypossibledigit]  = true;
				if (bYContains[y][onlypossibledigit]) m_bIsConsistent = false; // already has one!
				bYContains[y][onlypossibledigit]  = true;
				if (bSquareContains[SquareNumber(x,y)][onlypossibledigit]) m_bIsConsistent = false; // square already has one!
				bSquareContains[SquareNumber(x,y)][onlypossibledigit] = true;
			}
			if (numpossible == 0) {
				m_iKnownValues[x][y] = DIGIT_IMPOSSIBLE;
				m_bIsConsistent = false;
			}
			if (numpossible > 1) {
				m_bIsComplete = false;
			}
		}
	}
}

// Values in solution array

int CSuDoku::GetValueXY(int x, int y) // x and y in the range 0-8; returns 1-9 or -1 for unknown
{
	if (x<0 || x>8 || y<0 || y>8) return DIGIT_UNKNOWN;
	return m_iKnownValues[x][y];
}

int CSuDoku::GetValueYX(int y, int x) // x and y in the range 0-8; returns 1-9 or -1 for unknown
{
	return GetValueXY(x, y);
}

unsigned char CSuDoku::GetValueXYAsChar(int x, int y)
{
	if (x<0 || x>8 || y<0 || y>8) return ' ';
	if (m_iKnownValues[x][y]==DIGIT_UNKNOWN) return ' ';
	return m_iKnownValues[x][y] + '0';
}

unsigned char CSuDoku::GetValueYXAsChar(int y, int x)
{
	return GetValueXYAsChar(x, y);
}

// Values in starting array

int CSuDoku::GetStartingValueXY(int x, int y) // x and y in the range 0-8; returns 1-9 or -1 for unknown
{
	if (x<0 || x>8 || y<0 || y>8) return DIGIT_UNKNOWN;
	return m_iStartingValues[x][y];
}

int CSuDoku::GetStartingValueYX(int y, int x) // x and y in the range 0-8; returns 1-9 or -1 for unknown
{
	return GetValueXY(x, y);
}

unsigned char CSuDoku::GetStartingValueXYAsChar(int x, int y)
{
	if (x<0 || x>8 || y<0 || y>8) return ' ';
	if (m_iStartingValues[x][y]==DIGIT_UNKNOWN) return ' ';
	return m_iStartingValues[x][y] + '0';
}

unsigned char CSuDoku::GetStartingValueYXAsChar(int y, int x)
{
	return GetValueXYAsChar(x, y);
}

int CSuDoku::SquareNumber(int x, int y)
{
	// numbering of 3x3 squares:
	// 000 111 222
	// 000 111 222
	// 000 111 222
	// 333 444 555
	// 333 444 555
	// 333 444 555
	// 666 777 888
	// 666 777 888
	// 666 777 888

	// REMOVED FOR SPEED:
	// if (row<0 || row>8 || column<0 || column>8) return DIGIT_UNKNOWN;
	// int square = (row/3)*3 + (column/3); // integer (truncating) division should make this work
	// REMOVED FOR SPEED:
	// ASSERT(square>=0 && square<=8);
	// return square;

	return (y/3)*3 + (x/3);
}

void CSuDoku::SquareExtremes(int square, int& minx, int& maxx, int& miny, int& maxy)
{
	minx = (square%3)*3;
	maxx = minx+2;
	miny = (square/3)*3;
	maxy = miny+2;
}

bool CSuDoku::Solve(std::vector<CString>& vstrOutput)
{
	std::vector<CString> vstrRecursionAttempt;

	if (m_iRecursionLevel == 0 && !IsConsistent()) return false; // don't even bother

	ImproveAsMuchAsPossibleBySimpleScanning(vstrOutput);

	UpdateKnownValues(); // also updates m_bIsConsistent and m_bIsComplete
	if (!m_bIsConsistent) return false;

	if (!m_bIsComplete) { // Must adopt a more sophisticated strategy.
		CSuDoku copy;
		int x, y, d, numposs;

		// We could guess by iterating through x={0-8}, y={0,8} - but it takes ages.
		// What would be more efficient would be to guess cells with the least number of
		// possibles (minimum 2) first, and work up. Here's an easy way of doing that.

		int iNumPossible[9][9];
		for (x=0; x<9; ++x) {
			for (y=0; y<9; ++y) {
				if (m_bAllowedToGuess[x][y]) {
					iNumPossible[x][y] = 0;
					for (d=1; d<10; ++d) {
						if (m_bPossible[x][y][d]) {
							++iNumPossible[x][y];
						}
					}
				}
			}
		}

		for (numposs=2; numposs<10; ++numposs) {
			for (x=0; x<9; ++x) {
				for (y=0; y<9; ++y) {
					if (m_bAllowedToGuess[x][y] && iNumPossible[x][y]==numposs) {

						// OK. Now we've found a cell to guess.
						for (d=1; d<10; ++d) {
							if (m_bPossible[x][y][d]) {
								// Assume and recurse.
								copy = *this;
								vstrRecursionAttempt.clear();
								copy.Assume(x, y, d, vstrRecursionAttempt);
								if (copy.Solve(vstrRecursionAttempt)) {
									*this = copy;
									for (int i=0; i<vstrRecursionAttempt.size(); ++i) vstrOutput.push_back(vstrRecursionAttempt[i]);
									return true;
								}
							}
						}
						// Guesses for that cell complete.

					}
				}
			}
		}

	}

	UpdateKnownValues(); // also updates m_bIsConsistent and m_bIsComplete
	if (m_bIsComplete && m_bIsConsistent) return true;
	return false;
}

bool CSuDoku::ScanMatrixSimply(std::vector<CString>& vstrOutput)
{
	int masterX, masterY, square, digit, x, y, minx, maxx, miny, maxy;
	int nPossible, iPossibleX, iPossibleY;
	bool bImproved = false;

	// Begin by updating known values
	UpdateKnownValues();

	// Scan through all slots in the matrix. If a number is certain, wipe out all other possibilities
	// for that number in its row/column/square, and see if that solves any other cells.
	for (masterX=0; masterX<9; ++masterX) {
		for (masterY=0; masterY<9; ++masterY) {
			digit = m_iKnownValues[masterX][masterY];
			if (digit != DIGIT_UNKNOWN) {
				// Found one - wipe out all others.
				for (x=0; x<9; ++x) {
					for (y=0; y<9; ++y) {
						if ( (x == masterX && y != masterY) // same X, but not same cell
							 || (x != masterX && y == masterY) // same Y, but not same cell
							 || (SquareNumber(x,y) == SquareNumber(masterX,masterY) && !(x==masterX && y==masterY)) // same 3x3 square, but not same cell
							)
						{
							// We've found a cell that competes with our original cell, whose digit we know.
							// Apply restrictions to the cell, and see if it becomes solved as a result.
							RestrictAnythingBut(x, y, digit, vstrOutput, bImproved);
						}
					}
				}
			}
		}
	}

	// Scan through all numbers. If there's only one possibility for that number
	// in a row/column/square, confirm it (by eliminating other possible digits for that cell).
	for (digit=1; digit<10; ++digit) {
		// check through columns (defined by x value)
		for (x=0; x<9; ++x) {
			nPossible = 0;
			for (y=0; y<9; ++y) {
				if (m_bPossible[x][y][digit]) {
					++nPossible;
					iPossibleY = y;
				}
			}
			if (nPossible == 1) { // only one possible y in this x for this digit
				Restrict(x, iPossibleY, digit, vstrOutput, RESTRICTING_BY_X, bImproved);
			}
		}

		// check through rows (defined by y value)
		for (y=0; y<9; ++y) {
			nPossible = 0;
			for (x=0; x<9; ++x) {
				if (m_bPossible[x][y][digit]) {
					++nPossible;
					iPossibleX = x;
				}
			}
			if (nPossible == 1) { // only one possible x in this y for this digit
				Restrict(iPossibleX, y, digit, vstrOutput, RESTRICTING_BY_Y, bImproved);
			}
		}

		// check through squares
		for (square=0; square<9; ++square) {
			nPossible = 0;
			SquareExtremes(square, minx, maxx, miny, maxy);
			for (x=minx; x<=maxx; ++x) {
				for (y=miny; y<=maxy; ++y) {
					if (m_bPossible[x][y][digit]) {
						++nPossible;
						iPossibleX = x;
						iPossibleY = y;
					}
				}
			}
			if (nPossible == 1) { // only one possible cell in this square for this digit
				Restrict(iPossibleX, iPossibleY, digit, vstrOutput, RESTRICTING_BY_SQUARE, bImproved);
			}
		}
	}

	return bImproved;
}

bool CSuDoku::ScanMatrixAdvanced(std::vector<CString>& vstrOutput, int iGroupSizeForAdvancedStuff) // returns true iff improvement; iGroupSizeForAdvancedStuff = -1 or 2-8 (see code)
{
	int square, digit, x, y, minx, maxx, miny, maxy;
	bool bImproved = false;

	// MORE ADVANCED.

	// Begin by updating known values
	UpdateKnownValues();

	// Scan through all numbers in groups of n. If there are exactly n possible cells for each of those n
	// numbers in a row/column/square, and those possibilities are all the same,
	// eliminate other possible digits for those cells.
	// -- Example: in row 1, if digit 4 could be in columns 5/7 only, and digit 8 could be in columns 5/7 only,
	//    then either '4' goes in column 5 and '8' goes in column 7, or vice versa - but these cells can't
	//    possibly contain anything other than '4' or '8'.
	// We've done n=1 above, so we begin with n=2. No point going to n=9 (that means we have no
	// information!) or n=8 (if 8 wholly uncertain digits, then 1 completely certain digit, and that situation
	// was dealt with above, in ScanMatrixSimply).
	std::vector<int> viDigitsInGroup, viPossibleY[10], viPossibleX[10], viPossibleXToMatch, viPossibleYToMatch;
	// Pick every combination of digits from 1-9 in turn, except
	// those combinations with 0, 1, 8, or 9 digits in them.
	// Simplest way: cycle through a binary number where every bit represents the presence
	// of one digit (bit 0 = digit '1'; bit 1 = digit '2', and so on, up to bit 8 = digit '9').
	int counter, n, bit, digitcounter, i;
	bool bConditionsMet;
	for (counter = 1; counter <= (1<<9)-1; ++counter) {
		n =		((counter & (1<<0)) >> 0) +
				((counter & (1<<1)) >> 1) +
				((counter & (1<<2)) >> 2) +
				((counter & (1<<3)) >> 3) +
				((counter & (1<<4)) >> 4) +
				((counter & (1<<5)) >> 5) +
				((counter & (1<<6)) >> 6) +
				((counter & (1<<7)) >> 7) +
				((counter & (1<<8)) >> 8);
		// n is the number of bits set, i.e. the number of digits in this group to be tested
		if (n == 0 || n == 1 || n == 8 || n == 9) continue;
		if (iGroupSizeForAdvancedStuff != -1 && n != iGroupSizeForAdvancedStuff) continue;
		viDigitsInGroup.clear();
		for (bit = 0; bit < 9; ++bit) {
			if (counter & (1 << bit)) viDigitsInGroup.push_back(bit+1);
		}
		ASSERT(viDigitsInGroup.size() > 1);
		// OK. Now we have our group of n digits. Let's scan.

		// check through columns (defined by x value)
		for (x=0; x<9; ++x) {
			for (digit = 0; digit<10; ++digit) viPossibleY[digit].clear();
			for (digitcounter=0; digitcounter<viDigitsInGroup.size(); ++digitcounter) {
				// For every digit in our group, we want to know what possible Y coordinates (rows) it could be in.
				digit = viDigitsInGroup[digitcounter];
				for (y=0; y<9; ++y) {
					if (m_bPossible[x][y][digit]) {
						viPossibleY[digit].push_back(y);
					}
				}
			}
			viPossibleYToMatch = viPossibleY[viDigitsInGroup[0]];
			if (viPossibleYToMatch.size() == n) { // there are exactly n possible y-values for at least one of our digits
				bConditionsMet = true;
				for (digitcounter=1; digitcounter<viDigitsInGroup.size(); ++digitcounter) { // we know the zeroth one matches; check the rest
					digit = viDigitsInGroup[digitcounter];
					if (viPossibleY[digit] != viPossibleYToMatch) bConditionsMet = false;
				}
				if (bConditionsMet) {
					// OK! Conditions met. Restrict the cells in the possible columns to the list of digits.
					for (i=0; i<viPossibleYToMatch.size(); ++i) {
						RestrictToSet(x, viPossibleYToMatch[i], viDigitsInGroup, vstrOutput, RESTRICTING_BY_X, bImproved);
					}
				}
			}
		}

		// check through rows (defined by y value)
		for (y=0; y<9; ++y) {
			for (digit = 0; digit<10; ++digit) viPossibleX[digit].clear();
			for (digitcounter=0; digitcounter<viDigitsInGroup.size(); ++digitcounter) {
				// For every digit in our group, we want to know what possible Y coordinates (columns) it could be in.
				digit = viDigitsInGroup[digitcounter];
				for (x=0; x<9; ++x) {
					if (m_bPossible[x][y][digit]) {
						viPossibleX[digit].push_back(x);
					}
				}
			}
			viPossibleXToMatch = viPossibleX[viDigitsInGroup[0]];
			if (viPossibleXToMatch.size() == n) { // there are exactly n possible x-values for at least one of our digits
				bConditionsMet = true;
				for (digitcounter=1; digitcounter<viDigitsInGroup.size(); ++digitcounter) { // we know the zeroth one matches; check the rest
					digit = viDigitsInGroup[digitcounter];
					if (viPossibleX[digit] != viPossibleXToMatch) bConditionsMet = false;
				}
				if (bConditionsMet) {
					// OK! Conditions met. Restrict the cells in the possible x-values to the list of digits.
					for (i=0; i<viPossibleXToMatch.size(); ++i) {
						RestrictToSet(viPossibleXToMatch[i], y, viDigitsInGroup, vstrOutput, RESTRICTING_BY_Y, bImproved);
					}
				}
			}
		}

		// check through squares
		for (square=0; square<9; ++square) {
			// We're going to use a pair of identically-ordered arrays, to give us possible (x, y) locations.
			for (digit = 0; digit<10; ++digit) {
				viPossibleX[digit].clear();
				viPossibleY[digit].clear();
			}
			for (digitcounter=0; digitcounter<viDigitsInGroup.size(); ++digitcounter) {
				digit = viDigitsInGroup[digitcounter];
				SquareExtremes(square, minx, maxx, miny, maxy);
				for (x=minx; x<=maxx; ++x) {
					for (y=miny; y<=maxy; ++y) {
						if (m_bPossible[x][y][digit]) {
							viPossibleX[digit].push_back(x);
							viPossibleY[digit].push_back(y);
						}
					}
				}
			}
			viPossibleXToMatch = viPossibleX[viDigitsInGroup[0]];
			viPossibleYToMatch = viPossibleY[viDigitsInGroup[0]];
			ASSERT(viPossibleXToMatch.size() == viPossibleYToMatch.size());
			if (viPossibleXToMatch.size() == n) { // there are exactly n possible CELLS in this square for at least one of our digits
				bConditionsMet = true;
				for (digitcounter=1; digitcounter<viDigitsInGroup.size(); ++digitcounter) { // we know the zeroth one matches; check the rest
					digit = viDigitsInGroup[digitcounter];
					if (viPossibleX[digit] != viPossibleXToMatch) bConditionsMet = false;
					if (viPossibleY[digit] != viPossibleYToMatch) bConditionsMet = false;
				}
				if (bConditionsMet) {
					// OK! Conditions met. Restrict the cells in the possible rows to the list of digits.
					for (i=0; i<viPossibleXToMatch.size(); ++i) {
						RestrictToSet(viPossibleXToMatch[i], viPossibleYToMatch[i], viDigitsInGroup, vstrOutput, RESTRICTING_BY_SQUARE, bImproved);
					}
				}
			}
		}

	}

	return bImproved;
}

void CSuDoku::ImproveAsMuchAsPossibleBySimpleScanning(std::vector<CString>& vstrOutput)
{
	bool bImproved = true;
	while (bImproved && !IsComplete()) {
		bImproved = ScanMatrixSimply(vstrOutput);
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 2); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 3); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 4); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 5); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 6); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 7); // do advanced stuff for group-size 2
		if (!bImproved && !IsComplete()) bImproved = ScanMatrixAdvanced(vstrOutput, 8); // do advanced stuff for group-size 2
	}
}

void CSuDoku::Assume(int x, int y, int digit, std::vector<CString>& vstrOutput)
{
	for (int d=1; d<10; ++d) {
		if (d != digit) m_bPossible[x][y][d] = false;
	}
	m_bAllowedToGuess[x][y] = false; // prevents sub-iterations trying the same possibilities
	++m_iRecursionLevel;

	CString strMsg;
	strMsg.Format("Guessing x=%d, y=%d, digit=%d, recursion level=%d.", x+1, y+1, digit, m_iRecursionLevel);
	vstrOutput.push_back(strMsg);
}

void CSuDoku::Restrict(int x, int y, int digit, std::vector<CString>& vstrOutput, enum eRestrictions eRestrictingHow, bool& bImproved)
{
	bool bSolvedOne = false;
	for (int d=1; d<10; ++d) {
		if (d != digit && m_bPossible[x][y][d]) { // only allow changes from "possible" to "not possible"
			bSolvedOne = true;
			bImproved = true;
			m_bPossible[x][y][d] = false;
		}
	}
	if (bSolvedOne) {
		CString strMsg;
		switch (eRestrictingHow) {
		case RESTRICTING_BY_Y:
			strMsg.Format("In row y=%d, digit %d must go in column x=%d.", y+1, digit, x+1);
			break;
		case RESTRICTING_BY_X:
			strMsg.Format("In column x=%d, digit %d must go in row y=%d.", x+1, digit, y+1);
			break;
		case RESTRICTING_BY_SQUARE:
			strMsg.Format("In its square, digit %d must go in cell (x=%d, y=%d).", digit, x+1, y+1);
			break;
		default:
			ASSERT(FALSE);
			break;
		}
		vstrOutput.push_back(strMsg);
		TRACE(strMsg + "\n");
	}
}

void CSuDoku::RestrictAnythingBut(int x, int y, int digit, std::vector<CString>& vstrOutput, bool& bImproved)
{
	int iPreviouslyPossible = 0;
	int iNewlyPossible = 0;
	int iPossibleDigit;

	for (int d=1; d<10; ++d) {
		if (m_bPossible[x][y][d]) { // only allow things to change from "possible" to "not possible"
			++iPreviouslyPossible;
			++iNewlyPossible;
			if (d == digit) {
				// wipe out a value: our cell cannot be "digit".
				bImproved = true;
				m_bPossible[x][y][d] = false;
				--iNewlyPossible;
			}
			else {
				iPossibleDigit = d;
			}
		}
	}
	if (iNewlyPossible == 1 && iPreviouslyPossible > 1) {
		CString strMsg;
		strMsg.Format("Cell (x=%d, y=%d) now determined: can be nothing other than digit %d.", x+1, y+1, iPossibleDigit);
		vstrOutput.push_back(strMsg);
		TRACE(strMsg + "\n");
	}
}

void CSuDoku::RestrictToSet(int x, int y, std::vector<int> vPossibleDigits, std::vector<CString>& vstrOutput, enum eRestrictions eRestrictingHow, bool& bImproved)
{
	bool bPrevious[10];
	bool bNew[10];
	bool bImprovedLocally = false;

	for (int i=1; i<10; ++i) {
		bNew[i] = false;
	}
	for (i=0; i<vPossibleDigits.size(); ++i) {
		bNew[vPossibleDigits[i]] = true;
	}
	for (int d=1; d<10; ++d) {
		bPrevious[d] = m_bPossible[x][y][d];
		if (bPrevious[d] && !bNew[d]) { // only allow changes from "possible" to "not possible"
			m_bPossible[x][y][d] = bNew[d];
			bImproved = bImprovedLocally = true;
		}
	}
	if (bImprovedLocally) {
		CString strDigits, strPiece, strMsg;
		for (i=0; i<vPossibleDigits.size(); ++i) {
			if (i==0) strPiece.Format("%d", vPossibleDigits[i]);
			else strPiece.Format(",%d", vPossibleDigits[i]);
			strDigits += strPiece;
		}
		switch (eRestrictingHow) {
		case RESTRICTING_BY_Y:
			strMsg.Format("In row y=%d, one of digits %s must go in column x=%d.", y+1, strDigits, x+1);
			break;
		case RESTRICTING_BY_X:
			strMsg.Format("In column x=%d, one of digits %s must go in row y=%d.", x+1, strDigits, y+1);
			break;
		case RESTRICTING_BY_SQUARE:
			strMsg.Format("In its square, one of digits %s must go in cell (x=%d, y=%d).", strDigits, x+1, y+1);
			break;
		default:
			ASSERT(FALSE);
			break;
		}
		vstrOutput.push_back(strMsg);
		TRACE(strMsg + "\n");
	}
}


